//gnu.cpp - definition of the methods of Gnu
/*
 * Copyright (C) 2003 Lukas Fraser <lukas.fraser@4cows.net>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 */

#include <gnu.h>

//-------------------------------------------------------------
/*
 * constructor of Gnu
 *
 * @param gnupics all gnu qcanvaspixmaparrays (animations)
 * @param levelpics the level pictures (mask + pic)
 * @param canvas parent canvas
 * @param selected_type active gnu type when assigning it to gnu
 */

Gnu::Gnu(GnuPics * gnupics, LevelPics * levelpics, QCanvas * canvas, QString * selected_type, GnuzConfig * config):QCanvasSprite(gnupics->getPics("Walker0"), canvas)
{
  countdown_ = new QCanvasSprite(gnupics->getPics("Countdown"), canvas);
  countdown_->setVisible(FALSE);

  levelpics_=levelpics;
  gnupics_=gnupics;
  canvas_=canvas;
  selected_type_=selected_type;
  config_=config;

  resetGnu();
}

//-------------------------------------------------------------
/*
 * destructor of Gnu
 */

Gnu::~Gnu()
{
}

//-------------------------------------------------------------
/*
 * Reset Gnu params
 */

void Gnu::resetGnu()
{
  fall_=0;
  fall_pixel_=1;
  orientation_=1;
  type_="Walker";
  fast_mode_=FALSE;
  blocking_=FALSE;
  climbing_=FALSE;
  is_floater_=FALSE;
  is_climber_=FALSE;
  explode_active_=FALSE;
  explode_countdown_=100;
  advance_temp_=0;
  animate_skin_="";
  last_digged_=0;
  last_bashed_=0;
  built_steps_=0;
  spawninginside_=FALSE;
  spawnypos_=0;
  climb_reset_=TRUE;

  for (int i=0;i<config_->getGnuMoveIn();i++)
    climblist_[i]=0;

  setSkin(type_);
  setFrame(rand() % gnupics_->getPics(skin_)->count());

  setVelocity(orientation_, 0.0);
  show();
  countdown_->setVisible(FALSE);

  setVisible(TRUE);
  setActive(TRUE);
}

//-------------------------------------------------------------
/*
 * advance to next frame in animation (only every 2nd step) and check movement
 *
 * @param s
 */

void Gnu::advance(int s)
{
  if ((visible())||(spawninginside_))
  {
    //setFrameAnimation() substitution
    if (frameCount())
    {
      if (advance_temp_) //only animate every 2nd advance
      { 
	if (frame()<frameCount()-1)
	  setFrame(frame()+1);
	else setFrame(0);
	advance_temp_=0;
      }
      else
	advance_temp_=1;
    }

    QCanvasSprite::advance(s);
    if (s)
    {
      performAction();
      if (explode_active_)
	performCountdown();
    }

    if (fast_mode_)
    {
      QCanvasSprite::advance(s);
      if (s)
      {
	performAction();
	if (explode_active_)
	{
	  countdown_->advance(s);
	  performCountdown();
	}
      }
    }
  }
}

//-------------------------------------------------------------
/*
 * Gnu clicked -> change type
 *
 * @return true if type changed
 */

bool Gnu::clicked()
{
  if (!(selected_type_->isEmpty())&&(type_!="KilledGnu"))
  {
    setZ(100); //FIXXXME

    if (*selected_type_=="Bomber")
    {
      if (!explode_active_)
      {
	explode_active_=TRUE;
	activateCountdown();
	return(TRUE);
      }
    }

    if (*selected_type_=="Floater")
    {
      if (!is_floater_)
      {
	is_floater_=TRUE;
	return(TRUE);
      }
      else
	return(FALSE);
    }

    if (*selected_type_=="Climber")
    {
      if (!is_climber_)
      {
	is_climber_=TRUE;
	return(TRUE);
      }
      else
	return(FALSE);
    }
    
    if ((*selected_type_!=type_)&&(type_!="Blocker"))
    {
      if (*selected_type_=="Basher")
	resetClimb();
      
      last_digged_=0;
      built_steps_=0;
      if (orientation_)
	last_bashed_=levelpics_->getLevelWidth();
      else
	last_bashed_=0;

      type_=*selected_type_;

      last_built_y_=0;

      return(TRUE);
    }
  }
  return(FALSE);
}

//-------------------------------------------------------------
/*
 * Show Animation
 *
 * @param animation new animation
 */

bool Gnu::showAnimation(QString animation)
{
  if (!animation.isEmpty())
    setSkin(animation);

  if (frame()==frameCount()-1)
    return(FALSE);
 
  return(TRUE);
}

//-------------------------------------------------------------
/*
 * If Gnu was turned into a bomber (explode_active_==TRUE), perform countdown, set countdown sprite visible etc.
 */

void Gnu::performCountdown()
{
  countdown_->move(x()+((width()-gnupics_->getNumberWidth())/2), y()-(gnupics_->getNumberHeight()));

  explode_countdown_--;

  if (!explode_countdown_)
  {
    countdown_->setVisible(FALSE);
    kill("Bomber");
  }

  if ((explode_countdown_<(1000/config_->getNormalGnuSpeed())*9)&&(countdown_->visible()))
    countdown_->setFrame(8-(explode_countdown_/(1000/config_->getNormalGnuSpeed())));
}

//-------------------------------------------------------------
/*
 * update Skin
 *
 * @param skin
 */

void Gnu::setSkin(QString skin)
{
  char tempskin[30];

  sprintf(tempskin, "%s%d", skin.latin1(), orientation_);

  if (tempskin!=skin_)
  {
    setFrame(0);
    setSequence(gnupics_->getPics(tempskin));
    skin_=tempskin;

    if (skin=="Walker")
      setFrame(rand() % gnupics_->getPics(skin_)->count());

  }
}

//-------------------------------------------------------------
/*
 * activate countdown sprite above Gnu (Bomber)
 */

void Gnu::activateCountdown()
{
  countdown_->setZ(200);

  if (visible())
    countdown_->setVisible(TRUE);

  countdown_->move(x()+((width()-gnupics_->getNumberWidth())/2), y()-(gnupics_->getNumberHeight()));
  countdown_->setFrame(9);
}

//-------------------------------------------------------------
/*
 * change orientation_
 */

void Gnu::turnAround()
{
  if (type_!="KilledGnu")
  {
    if (orientation_)
      orientation_=0;
    else
      orientation_=1;
    
    setXVelocity(0.0);
    
    if ((type_=="Builder")||(type_=="Basher")) //Stop building/bashing
      type_="Walker";
    
    resetClimb();
  }
}


//-------------------------------------------------------------
/*
 * Handle Gnu actions, depending on type_, is_floater_, and is_climber_
 */

void Gnu::performAction()
{
  if (type_=="Walker")       { walk(1.0, type_); }
  if (type_=="Blocker")      { block(); walk(0.0, type_, FALSE); }
  if (type_=="Digger")       { dig(); walk(0.0, type_); }
  if (type_=="Basher")       { bash(); walk(0.3, type_); }
  if (type_=="Builder")      { build(); walk(0.3, type_); }
  if (type_=="KilledGnu")    { kill(); }
}

//-------------------------------------------------------------
/*
 * Check movement: walk, climb, fall etc.
 *
 * @param speed x velocity
 * @param skin
 * @param climb_up if true, gnu can walk up the hill
 */

void Gnu::walk(double speed, QString skin, bool climb_up)
{
  if ((type_!="KilledGnu")&&(isOnFloor()||(yVelocity()<0)))
  {
    if ((fall_pixel_>=config_->getGnuMaxFallHeight())&&(!is_floater_))
      kill("Drop");
    else
    {
      if (fall_)
      {
	fall_=0;	

	resetClimb();

	if (is_climber_)
	  climb();
      }

      if (orientation_) setXVelocity(speed);
      else setXVelocity(-(speed));
	
      setYVelocity(0.0);
      if (!climbing_)
	setSkin(skin);
      
      if ((!isMovePossible())&&(xVelocity()))
      {
	if ((is_climber_)&&((int)y()>0)&&((int)x()>1)&&((int)x()+width()<levelpics_->getLevelWidth()-1))
	{
	  climb_reset_=TRUE;
	  climblist_[config_->getGnuMoveIn()-1]=(int)y()+3;
	  climb();
	  if (isMovePossible())
	  {
	    if (orientation_) moveBy(speed, 0.0); //FIXXME
	    else moveBy(-(speed), 0.0);

	    climbing_=FALSE;
	    return;
	  }
	  else
	    climblist_[config_->getGnuMoveIn()-1]=-1;

	  int tempxpos;
	  if (orientation_) tempxpos=(int)x()+width();
	  else tempxpos=(int)x()-1;

	  if ((!levelpics_->isOtherColorAtLevelMask((int)x(), (int)y(), width(), 1, QColor(white), QColor(darkRed))))
	  {
	    setVelocity(0.0, -1.0);
	    setSkin("Climber");
	    climbing_=TRUE;
	  }
	  else
	  {
	    climbing_=FALSE;
	    fall_=4; //FIXXME... don't start walking in the air
	    turnAround();
	  }
	}
	else
	  turnAround();
      }
      else
      {
	if ((climb_up)&&(x()>0)&&(x()+width()<=levelpics_->getLevelWidth()))
	  climb();
	climbing_=FALSE;
      }
    }
  }
  else
    fall();
}

//-------------------------------------------------------------
/*
 * bash through walls, hills...
 */

void Gnu::bash()
{
  int bashed=0;
  int bashed_special=0;
  int tempxpos;
  int tempypos=(int)y();

  if (isOnFloor())
  {
    if (type_=="Basher")
    {
      if (orientation_) tempxpos=(int)x()+width()+1;
      else tempxpos=(int)x()-2;
      
      if ((orientation_)&&(last_bashed_==levelpics_->getLevelWidth()))
      {
	bashed=levelpics_->drawRectLevel(tempxpos-width(), (int)y(), width(), height(), QColor(white), QColor(black), QColor(black));
	bashed_special=levelpics_->drawRectLevel(tempxpos-width(), (int)y(), width(), height(), QColor(white), QColor(cyan), QColor(black));
      }
      if ((!orientation_)&&(last_bashed_==0))
      {
	bashed=levelpics_->drawRectLevel(tempxpos+1, (int)y(), width(), height(), QColor(white), QColor(black), QColor(black));
	bashed_special=levelpics_->drawRectLevel(tempxpos+1, (int)y(), width(), height(), QColor(white), QColor(magenta), QColor(black));
      }

      bashed=levelpics_->drawRectLevel(tempxpos, tempypos, 1, height(), QColor(white), QColor(black), QColor(black));
      
      if (orientation_)
	bashed_special=levelpics_->drawRectLevel(tempxpos, tempypos, 1, height(), QColor(white), QColor(cyan), QColor(black));
      else
	bashed_special=levelpics_->drawRectLevel(tempxpos, tempypos, 1, height(), QColor(white), QColor(magenta), QColor(black));
      
      if (levelpics_->isOtherColorAtLevelMask(tempxpos, tempypos, 1, height(), QColor(white)))
      {
	bashed=0;
	bashed_special=0;
      }
      
      if ((bashed)||(bashed_special))
	last_bashed_=tempxpos;
      else //Stop bashing
      {
	if (orientation_)
	{
	  if (tempxpos>last_bashed_)
	    type_="Walker";
	}
	else
	{
	  if (tempxpos<last_bashed_)
	    type_="Walker";
	}
      }
    }
  }
}

//-------------------------------------------------------------
/*
 * dig in the ground
 */

void Gnu::dig()
{
  int digged=0;
  int tempxpos=(int)x();
  int tempypos=(int)y()+height();

  if (isOnFloor())
  {
    if ((fall_pixel_>=config_->getGnuMaxFallHeight())&&(!is_floater_))
      kill("Drop");

    if (type_=="Digger")
    {
      if (last_digged_==0)
	digged=levelpics_->drawRectLevel((int)x(), (int)y(), width(), height(), QColor(white), QColor(black), QColor(black));
      
      digged=levelpics_->drawRectLevel(tempxpos, tempypos, width(), 1, QColor(white), QColor(black), QColor(black));
      
      if ((digged)&&(!levelpics_->isOtherColorAtLevelMask(tempxpos, tempypos, width(), 1, QColor(white))))
      {
	setSkin(type_);
	last_digged_=tempypos;
      }
      else //Stop digging
      {
	if (tempypos>last_digged_)
	{
	  resetClimb();
	  
	  type_="Walker";
	  setSkin(type_);
	}
      }
    }
    else
    {
      if ((tempypos>last_digged_)&&(last_digged_!=0))
      {
	resetClimb();
	
	type_="Walker";
	setSkin(type_);
      }
    }
  }
  else
  {
    if ((tempypos>last_digged_)&&(last_digged_!=0))
    {
      resetClimb();
      
      type_="Walker";
      setSkin(type_);
    }
  }
}

//-------------------------------------------------------------
/*
 * Blocker
 *
 * @param unblock if true -> stop blocking
 */

void Gnu::block(int unblock)
{
  if ((!blocking_)||(unblock))
  {
    if (isOnFloor())
    {
      turnAround();

      if (!blocking_) //start blocking
	blocking_=levelpics_->drawRectLevel((int)x(), (int)y(), width(), height(), QColor(darkRed), QColor(white), QColor());

      if (unblock) //stop blocking
	blocking_=levelpics_->drawRectLevel((int)x(), (int)y(), width(), height(), QColor(white), QColor(darkRed), QColor());
    }
  }
}

//-------------------------------------------------------------
/*
 * Builder
 */

void Gnu::build()
{
  int built=0;
  int build=0;
  int stop=0;
  int tempxpos;
  int tempypos=(int)y()+height()-1;

  if ((int)y()<=1) //Don't build too high
    stop=1;

  if ((isOnFloor())&&(type_=="Builder"))
  {
    if (orientation_) 
    {
      tempxpos=(int)x()+width();
      if (tempxpos==last_built_x_+2)
      {
	build=1;
	tempypos=last_built_y_-1;
      }
    }
    else
    {
      tempxpos=(int)x()-2;
      if (tempxpos==last_built_x_-2)
      {
	build=1;
	tempypos=last_built_y_-1;
      }
    }
    /*
      if (tempypos+1<last_built_y_)
      stop=1;
    */
    //Build base
    if (!built_steps_)
    {
      if (isOnFloor())
      {    
	if (!levelpics_->isColorAtLevelMask((int)x(), (int)y()+height()-1, width()+2, 2, QColor(yellow)))
	{
	  if (orientation_)
	    buildBase(tempxpos, tempypos+1);
	  else
	    buildBase(tempxpos, tempypos+1);
	}
	
	build=1;
      }
      else
	stop=1;
    } 
    
    if ((built_steps_<config_->getGnuBuilderMaxSteps())&&(build)&&(!stop))
    {
      if (!levelpics_->isOtherColorAtLevelMask(tempxpos, (int)y()-2, 1, height(), QColor(white), QColor(yellow)))
      {
	built=levelpics_->drawRectLevel(tempxpos, tempypos, 4, 2, QColor(yellow), QColor(white), QColor(yellow));
	
	if (built)
	{
	  setSkin("Builder");
	  built_steps_++;
	  last_built_x_=tempxpos;
	  last_built_y_=tempypos;
	  
	  levelpics_->getLevelSprite()->setFrame(0); //FIXXME... without this, picture doesn't get updated 100%...
	}
	else
	  stop=1;
      }
      else
	stop=1;
    }
    else
    {
      if (built_steps_==config_->getGnuBuilderMaxSteps())
	stop=1;
    }
  }

  if (stop)
  {
    type_="Walker";
    built_steps_=0;
    last_built_y_=levelpics_->getLevelHeight()-1;
  }
}

//-------------------------------------------------------------
/*
 * build base for Builder
 *
 * @param x coord
 * @param y coord
 */

void Gnu::buildBase(int xpos, int ypos)
{
  int built;
  int tempsize;
  
  if (orientation_)
  {
    tempsize=(-(2));
    if (xpos+(tempsize*2)<(int)x()+(width()/2))
      return;
  }
  else
  {
    tempsize=2;
    if (xpos+(tempsize*2)>(int)x()+(width()/2))
      return;
  }

  if(!levelpics_->isOtherColorAtLevelMask(xpos+(tempsize*2), ypos+1, 4, 2, QColor(white)))
  {
    built=levelpics_->drawRectLevel(xpos+tempsize, ypos, 4, 2, QColor(yellow), QColor(white), QColor(yellow));

    if (built) //Continue builing a base
      buildBase(xpos+tempsize, ypos+1);
  }
}

//-------------------------------------------------------------
/*
 * Climb
 */

void Gnu::climb()
{
  int tempxposstart;
  int tempxposend;
  int tempyposstart=(int)y();
  int tempyposend=(int)y()+height();
  int tempclimbheight;

  if (orientation_)
    tempxposstart=(int)x()+width()+1;
  else
    tempxposstart=(int)x()-2;

  if (climb_reset_)
  {
    if (orientation_)
    {
      tempxposend=tempxposstart;
      tempxposstart=tempxposend-1;
    }
    else
      tempxposend=tempxposstart+1;
  }
  else
    tempxposend=tempxposstart;

  if (((int)y()-climblist_[0]>0)&&((int)y()-climblist_[0]<=3)&&(climblist_[0]>0))
    setYVelocity(-((int)y()-climblist_[0]));

  climb_reset_=FALSE;

  //Climb up
  if (tempyposend<=levelpics_->getLevelHeight()+1)
  {
    if ((tempxposstart>0)&&(tempxposend<=levelpics_->getLevelWidth()))
    {
      for (int i=tempxposstart;i<=tempxposend;i++)
      {
	for (int k=0;k<config_->getGnuMoveIn()-1;k++)
	  climblist_[k]=climblist_[k+1];

	for (int j=tempyposstart;j<tempyposend;j++)
	{
	  QRgb pixel = levelpics_->getLevelMask()->pixel(i, j);
	  
	  if (QColor(pixel)!=white)
	  {
	    if (QColor(pixel)==darkRed)
	    {
	      turnAround();
	      return;
	    }
	    
	    if (!((QColor(pixel)==yellow)&&(tempyposstart-(tempyposend-j)<climblist_[config_->getGnuMoveIn()-2]-1)))
	    {
	      tempclimbheight=tempyposstart-(tempyposend-j);
	      
	      if (climblist_[config_->getGnuMoveIn()-2]-tempclimbheight<=3)
		climblist_[config_->getGnuMoveIn()-1]=tempclimbheight;
	      else
		climblist_[config_->getGnuMoveIn()-1]=-1;
	      return;
	    }
	  }
	}
      }
    }
  }
  else
    climblist_[config_->getGnuMoveIn()-1]=-1;
  
  climblist_[config_->getGnuMoveIn()-1]=tempyposstart;
}

//-------------------------------------------------------------
/*
 * Reset Climb array
 */

void Gnu::resetClimb()
{
  for (int i=0;i<config_->getGnuMoveIn();i++)
    climblist_[i]=(int)y();

  climb_reset_=TRUE;
}

//-------------------------------------------------------------
/*
 * Fall
 */

void Gnu::fall()
{
  if (spawninginside_)
  {
    setVelocity(0.0, 1.0);
    setSkin("Fall");
    return;
  }

  if (!fall_)
  {
    fall_lasty_=(int)y();
    fall_pixel_=0;
  }

  fall_++;
  fall_pixel_+=(int)y()-fall_lasty_;
  fall_lasty_=(int)y();

  if ((type_=="Digger")&&((last_digged_!=0)))
  {
    fall_=0;
    fall_pixel_=0;
    setVelocity(0.0, 0.2);
    return;
  }

  if ((int)y()+height()>=0)
  {
    if (type_=="Basher")
    {
      if ((last_bashed_==0)||(last_bashed_==levelpics_->getLevelWidth()))
      {
	if (xVelocity())
	  type_="Walker";
      }
      else
      {
	if (fall_>1)
	  type_="Walker"; //Stop Bashing if falling down from somewhere	  
      }
    }

    if (!is_floater_)
    {
      setVelocity(0.0, getFallSpeed());
      if (fall_>2)
	setSkin("Fall");
    }
    else
    {
      setVelocity(0.0, 1.0);
      if (fall_>3)
	setSkin("Floater");
    }
  }
  else
  {
    fall_=0;
    fall_pixel_=0;
    setVelocity(0.0, 1.0);
    setSkin("Fall");
  }
}

//-------------------------------------------------------------
/*
 * Kill Gnu
 *
 * @param skin drown, drop dead...
 */

void Gnu::kill(QString skin)
{
  setVelocity(0.0, 0.0);

  if ((animate_skin_.isEmpty())&&(!skin.isEmpty())&&(type_!="KilledGnu"))
  {
    animate_skin_=skin;
    type_="KilledGnu";
  }

  if (animate_skin_.isEmpty())
  {
    countdown_->setVisible(FALSE);
    setVisible(FALSE);

    if (active())
      gnusleft_->setNum(gnusleft_->getNum()-1);

    setActive(FALSE);
  }

  if (!showAnimation(animate_skin_)) //Animate till end of animation, then stop
  {
    if (animate_skin_=="Bomber")
    {
      int radius=width();
      int xposstart=(int)x()+(width()/2)-radius;
      int yposstart=(int)y()+(height()/2);
      
      if (blocking_) block(1);
      
      if (visible())
      {
	levelpics_->drawCircleLevel(xposstart, yposstart, radius, QColor(white), QColor(black), QColor(black));
	levelpics_->getLevelSprite()->setFrame(0); //FIXXME... without this, picture doesn't get updated 100%...
      }
    }
    animate_skin_="";
  }
}

//-------------------------------------------------------------
/*
 * Reach Exit
 */

void Gnu::reach_exit()
{
  gnussaved_->setNum(gnussaved_->getNum()+1);
  kill("Exit");
}





//-------------------------------------------------------------
/*
 * Calc fall speed of Gnu
 *
 * @return fall speed
 */

double Gnu::getFallSpeed()
{
  double tempspeed=1+(fall_/20)*(fall_/20);
  int tempxposstart=(int)x()+1;
  int tempxposend=(int)x()+width()-2;
  int tempyposstart=(int)y()+height();
  int tempyposend=tempyposstart+(int)tempspeed;

  if (tempxposend>=levelpics_->getLevelWidth()-1) tempxposend=levelpics_->getLevelWidth()-1;
  if (tempyposend>=levelpics_->getLevelHeight()-1)
  {
    tempyposend=levelpics_->getLevelHeight()-1;
    tempspeed=tempyposend-tempyposstart+1;
  }
 
  for (int i=tempyposstart;i<tempyposend;i++)
  {
    for (int j=tempxposstart;j<=tempxposend;j++)
    {
      QRgb pixel = levelpics_->getLevelMask()->pixel(j, i);
      if ((QColor(pixel)!=white)&&(QColor(pixel)!=darkRed))
	return(i-tempyposstart);
    }
  }
  return(tempspeed);
}

//-------------------------------------------------------------
/*
 * Check if you gnu can move to left/right (depending on orientation_)
 *
 * @return true if Gnu can move on
 */

bool Gnu::isMovePossible()
{
  int tempxpos;

  if (orientation_)
    tempxpos=(int)x()+width();
  else
    tempxpos=(int)x();

  if ((!orientation_)&&(tempxpos<0))
    return(FALSE);
  if ((orientation_)&&(tempxpos>levelpics_->getLevelWidth()+1))
      return(FALSE);

  if (x()-1<0) return(TRUE);
  if (x()>levelpics_->getLevelWidth()) return(TRUE);
  
  if ((climblist_[config_->getGnuMoveIn()-1]>=0)&&(climblist_[config_->getGnuMoveIn()-2]>=0)&&(climblist_[config_->getGnuMoveIn()-2]-climblist_[config_->getGnuMoveIn()-1]<=3))
    return(TRUE);

  return(FALSE);
}

//-------------------------------------------------------------
/*
 * Check if you gnu is on the floor, kill if on water, or reaches exit
 *
 * @return true if on floor
 */

bool Gnu::isOnFloor()
{
  int tempxpos=(int)x();
  int tempypos=(int)y()+height();

  if (spawninginside_)
  {
    if ((int)y()>=spawnypos_)
    {
      setVisible(1);
      if (explode_active_)
	countdown_->setVisible(TRUE);
      spawninginside_=FALSE;
    }
    else
      return(FALSE);
  }

  if ((tempxpos<0)||(tempxpos+width()>levelpics_->getLevelWidth()))
    return(TRUE);
  else
  {
    if (tempypos>=levelpics_->getLevelHeight())
      return(TRUE);

    if (tempypos<0)
      return(FALSE);
  }
  
  if (type_=="KilledGnu")
    return(TRUE);

  if (levelpics_->isOtherColorAtLevelMask(tempxpos, tempypos, width(), 1, QColor(white), QColor(darkRed)))
  {
    if (!levelpics_->isOtherColorAtLevelMask(tempxpos, tempypos, width(), 1, QColor(blue))) //Drown when on water
    {
      kill("Drown");
      return(TRUE);
    }
    if (!levelpics_->isOtherColorAtLevelMask(tempxpos, tempypos, width(), 1, QColor(green))) //Exit level
    {
      reach_exit();
      return(TRUE);
    }

    return(TRUE);
  }

  return(FALSE);
}
