//gnu.h - declaration of the class Gnu
/*
 * Copyright (C) 2003 Lukas Fraser <lukas.fraser@4cows.net>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 */

#ifndef __gnu_h__
#define __gnu_h__

#include <qapplication.h>
#include <qcanvas.h>
#include <qimage.h>
#include <qpainter.h>
#include <levelpics.h>
#include <math.h>
#include <qmap.h>
#include <gnupics.h>
#include <gnutypeselectlabel.h>
#include <gnuzconfig.h>
#include <stdlib.h>

//----------------------------------------------------------
/*
 * Gnu
 * 
 * Inherits QCanvasSprite
 *
 */

static const int gnuRTTI = 1234;

class Gnu : public QCanvasSprite
{
 protected:
  QMap<int, int> climblist_;
  QCanvasSprite *countdown_;
  GnuPics *gnupics_;
  LevelPics *levelpics_;
  QCanvas *canvas_;
  GnuTypeSelectLabel *gnusleft_;
  GnuTypeSelectLabel *gnussaved_;
  int orientation_;
  int fall_;
  int fall_pixel_;
  int fall_lasty_;
  int last_bashed_;
  int last_digged_;
  int last_built_x_;
  int last_built_y_;
  bool blocking_;
  bool climbing_;
  bool is_floater_;
  bool is_climber_;
  bool explode_active_;
  int explode_countdown_;  
  QString type_;
  QString *selected_type_;
  QString skin_; 
  int advance_temp_;
  int built_steps_;
  QString animate_skin_;
  bool spawninginside_;
  int spawnypos_;
  bool fast_mode_;
  GnuzConfig *config_;
  bool climb_reset_;

 public:
  Gnu(GnuPics * gnupics, LevelPics * levelpics, QCanvas * canvas, QString * selected_type, GnuzConfig * config);
  virtual ~Gnu();

  void resetGnu();
  void advance(int s);
  bool clicked();
  bool showAnimation(QString animation);
  void setSkin(QString skin);
  void activateCountdown();
  void performCountdown();
  void turnAround();

  void performAction();
  void walk(double speed, QString skin, bool climb_up=TRUE);
  void bash();
  void dig();
  void block(int unblock=0);
  void build();
  void buildBase(int xpos, int ypos);
  void climb();
  void resetClimb();
  void fall();
  void kill(QString skin="");
  void reach_exit();

  double getFallSpeed();
  bool isMovePossible();
  bool isOnFloor();

  void setFastMode(bool fast) { fast_mode_=fast; }
  void setExplodeCountdown(int countdown) { explode_countdown_=countdown; }
  void setGnusLeftCounter(GnuTypeSelectLabel *gnusleft) { gnusleft_=gnusleft; }
  void setGnusSavedCounter(GnuTypeSelectLabel *gnussaved) { gnussaved_=gnussaved; }
  void setOrientation(int orientation) { orientation_=orientation; }
  void setSpawningInsideLevel(bool spawninginside, int spawnypos) { spawninginside_=spawninginside; spawnypos_=spawnypos; setVisible(FALSE); }
  int rtti () const { return gnuRTTI; }
};

#endif // __gnu_h__
