//gnupics.cpp - definition of the methods of GnuPics
/*
 * Copyright (C) 2003 Lukas Fraser <lukas.fraser@4cows.net>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 */

#include <gnupics.h>

//-------------------------------------------------------------
/*
 * constructor of GnuPics
 */

GnuPics::GnuPics(GnuzConfig *config)
{
  config_=config;

  gnupics_.clear();

  picsdata_ = new GnuzData(config_->getPicsDir()+"gnuimages.dat");

  mirror("Basher", 4, TRUE);
  mirror("Blocker", 1, TRUE);
  mirror("Bomber", 5, TRUE);
  mirror("Builder", 5, TRUE);
  mirror("Climber", 3, TRUE);
  mirror("Countdown", 10, FALSE);
  mirror("Digger", 4, TRUE);
  mirror("Drop", 6, TRUE);
  mirror("Drown", 7, TRUE);
  mirror("Exit", 8, TRUE);
  mirror("Fall", 1, TRUE);
  mirror("Floater", 3, TRUE);
  mirror("Walker", 8, TRUE);
}

//-------------------------------------------------------------
/*
 * destructor of GnuPics
 */

GnuPics::~GnuPics()
{
}

//-------------------------------------------------------------
/*
 * Reads Pics from gnuimages.dat, adds them to gnupics_, and creates a mirrored copy if mirror==TRUE
 *
 * @param name name of the animation
 * @param framecount number of frames the animation has
 * @param mirror if true, create a mirrored copy
 */

void GnuPics::mirror(QString name, int framecount, bool mirror)
{
  QList<QPixmap> picslist;
  QList<QPixmap> mirrored_picslist;
  QList<QPoint> pointlist;
  QPoint temppoint;
  QString tempname;

  QImage load_image;
  QImage mirrored_image;
  QPixmap load_pixmap;
  
  for (int i=0;i<framecount;i++)
  {
    load_image=picsdata_->getImage();
    load_pixmap.convertFromImage(load_image);
    
    picslist.append(new QPixmap(load_pixmap));
    pointlist.append(&temppoint);

    if (mirror)
    {
      mirrored_image=load_image.mirror(1, 0);
      mirrored_image.setAlphaBuffer(1);

      load_pixmap.convertFromImage(mirrored_image);
      mirrored_picslist.append(new QPixmap(load_pixmap));
    }
  }

  if (mirror)
    tempname=name+"1";
  else
    tempname=name;
  
  gnupics_.insert(tempname, new QCanvasPixmapArray(picslist, pointlist));

  if (mirror)
  {
    tempname=name+"0";
    gnupics_.insert(tempname, new QCanvasPixmapArray(mirrored_picslist, pointlist));
  }
}
