//gnutypeselectlabel.cpp - definition of the methods of GnuTypeSelectLabel
/*
 * Copyright (C) 2003 Lukas Fraser <lukas.fraser@4cows.net>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 */

#include <gnutypeselectlabel.h>

//-------------------------------------------------------------
/*
 * constructor of GnuTypeSelectLabel
 *
 * @param parent
 * @param min_value until value_ is lower than min_value->red font
 * @param value start value
 * @param emit_signal emits signal if value changed
 */

GnuTypeSelectLabel::GnuTypeSelectLabel(QWidget *parent, int min_value, int value, int emit_signal):QLabel(parent)
{
  min_value_=min_value;
  value_=value;
  emit_signal_=emit_signal;
  start_value_=value_;

  setFrameStyle(QFrame::Box | QFrame::Plain);
  setLineWidth(0);
  setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
  setFont(QFont("Cronyx-Helvetica", 12, QFont::Bold));
  setNum(value_);
}

//-------------------------------------------------------------
/*
 * change value and update color of text
 *
 * @param value new value
 */

void GnuTypeSelectLabel::setNum(int value)
{
  value_=value;

  QPalette pal=palette();

  if (min_value_==0)
  {
    if (value_>min_value_)
      pal.setColor(QColorGroup::Foreground, darkGreen);
    else
      pal.setColor(QColorGroup::Foreground, red);
  }
  else
  {
    if (value_>=min_value_)
      pal.setColor(QColorGroup::Foreground, darkGreen);
    else
      pal.setColor(QColorGroup::Foreground, red);
  }

  setPalette(pal);
  QLabel::setNum(value_);

  if (emit_signal_)
    emit valueChanged();
}
