//gnutypeselector.cpp - definition of the methods of GnuTypeSelector
/*
 * Copyright (C) 2003 Lukas Fraser <lukas.fraser@4cows.net>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 */

#include <gnutypeselector.h>

//-------------------------------------------------------------
/*
 * constructor of GnuTypeSelector
 *
 * @param parent
 */

GnuTypeSelector::GnuTypeSelector(QWidget *parent, GnuzConfig *config, GuiPics *guipics)
{
  config_=config;
  type_="Walker";
  killall_pressed_=0;
  guipics_=guipics;

  layout_ = new QGridLayout();
 
  buttons_.insert("Blocker", new GnuTypeSelectButton(guipics_->getPic("Blocker"), parent, "Blocker"));
  buttons_.insert("Digger", new GnuTypeSelectButton(guipics_->getPic("Digger"), parent, "Digger"));
  buttons_.insert("Basher", new GnuTypeSelectButton(guipics_->getPic("Basher"), parent, "Basher"));
  buttons_.insert("Builder", new GnuTypeSelectButton(guipics_->getPic("Builder"), parent, "Builder"));
  buttons_.insert("Floater", new GnuTypeSelectButton(guipics_->getPic("Floater"), parent, "Floater"));
  buttons_.insert("Climber", new GnuTypeSelectButton(guipics_->getPic("Climber"), parent, "Climber"));
  buttons_.insert("Bomber", new GnuTypeSelectButton(guipics_->getPic("Bomber"), parent, "Bomber"));

  buttons_.insert("KillAll", new GnuTypeSelectButton(guipics_->getPic("Bomb"), parent, "KillAll"));
  buttons_.insert("Pause", new GnuTypeSelectButton(guipics_->getPic("Pause"), parent, "Pause"));
  buttons_.insert("FastForward", new GnuTypeSelectButton(guipics_->getPic("FastForward_inactive"), parent, "FastForward"));

  label_numbers_.insert("GnusLeft", new GnuTypeSelectLabel(parent, 0, 0, 1));
  label_numbers_.insert("GnusSaved", new GnuTypeSelectLabel(parent, 0, 0));

  label_gnusleft_image_ = new QLabel(parent);
  label_gnusleft_image_->setPixmap(*guipics_->getPic("GnusLeft"));
  
  label_gnussaved_image_ = new QLabel(parent);
  label_gnussaved_image_->setPixmap(*guipics_->getPic("GnusSaved"));

  ButtonMap::Iterator it;
  for ( it = buttons_.begin(); it != buttons_.end(); ++it )
  {
    if ((it.key().latin1()!="KillAll")&&(it.key().latin1()!="Pause")&&(it.key().latin1()!="FastForward"))
    {
      label_numbers_.insert(it.key().latin1(), new GnuTypeSelectLabel(parent, 0, 0));

      if (!label_numbers_[it.key().latin1()]->getNum())
	buttons_[it.key().latin1()]->setDisabled(1);
    }

    QObject::connect(buttons_[it.key().latin1()], SIGNAL(newType(QString)), this, SLOT(changeType(QString)));
  }

  layout_->addWidget(buttons_["Blocker"], 0, 0);
  layout_->addWidget(buttons_["Digger"], 0, 1);
  layout_->addWidget(buttons_["Basher"], 0, 2);
  layout_->addWidget(buttons_["Builder"], 0, 3);
  layout_->addWidget(buttons_["Floater"], 0, 4);
  layout_->addWidget(buttons_["Climber"], 0, 5);
  layout_->addWidget(buttons_["Bomber"], 0, 6);
  
  layout_->addWidget(buttons_["FastForward"], 2, 4);
  layout_->addWidget(buttons_["Pause"], 2, 5);
  layout_->addWidget(buttons_["KillAll"], 2, 6);

  layout_->addWidget(label_numbers_["Blocker"], 1, 0);
  layout_->addWidget(label_numbers_["Digger"], 1, 1);
  layout_->addWidget(label_numbers_["Basher"], 1, 2);
  layout_->addWidget(label_numbers_["Builder"], 1, 3);
  layout_->addWidget(label_numbers_["Floater"], 1, 4);
  layout_->addWidget(label_numbers_["Climber"], 1, 5);
  layout_->addWidget(label_numbers_["Bomber"], 1, 6);

  layout_->addWidget(label_gnusleft_image_, 2, 0, Qt::AlignRight);
  layout_->addWidget(label_numbers_["GnusLeft"], 2, 1);
  layout_->addWidget(label_gnussaved_image_, 2, 2, Qt::AlignRight);
  layout_->addWidget(label_numbers_["GnusSaved"], 2, 3);
}

//-------------------------------------------------------------
/*
 * destructor of GnuTypeSelector
 */

GnuTypeSelector::~GnuTypeSelector()
{
  /*
  for (int i=0;i<(int)buttons_.size();i++)
    delete buttons_[i];

  for (int i=0;i<(int)label_numbers_.size();i++)
    delete label_numbers_[i];
  */
  delete guipics_;
  delete layout_;
}


//-------------------------------------------------------------
/*
 * SLOT: change active type of gnu and set button on
 *
 * @param type
 */

void GnuTypeSelector::changeType(QString type)
{
  if (type=="KillAll")
  {
    if (!killall_pressed_)
    {
      killall_pressed_=1;
      changeButtonPixmap("KillAll", "Bomb_activated");
      buttons_["KillAll"]->setOn(1);
    }
    else //KillAll command confirmed
    {
      if (type_!=type)
      {
	if (buttons_["Pause"]->isOn()) //Deactivate pause if pressed
	{
	  emit newType("Pause");
	  buttons_[type]->setOn(0);
	}
	type_=type;
	setButtonsDisabled(1);
	emit newType(type_);
      }
    }
    return;
  }
  else
  {
    killall_pressed_=0;

    changeButtonPixmap("KillAll", "Bomb");
    buttons_["KillAll"]->setOn(0);
    emit newType(type);
  }

  if ((type=="Pause")||(type=="FastForward"))
  {
    if (!buttons_[type]->isOn())
      buttons_[type]->setOn(0);
    else
      buttons_[type]->setOn(1);

    return;
  }

  ButtonMap::Iterator it;
  for ( it = buttons_.begin(); it != buttons_.end(); ++it )
  {
    if ((it.key()!="KillAll")&&(it.key()!="Pause")&&(it.key()!="FastForward"))
    {
      if (it.key().latin1()!=type)
	buttons_[it.key().latin1()]->setOn(0);
      else buttons_[it.key().latin1()]->setOn(1);
    }
  }
  type_=type;
}

//-------------------------------------------------------------
/*
 * SLOT: change type of gnu
 *
 * @param button
 * @param pixmap
 */

void GnuTypeSelector::changeButtonPixmap(QString button, QString pixmap)
{
  buttons_[button]->setPixmap(*guipics_->getPic(pixmap));
}

//-------------------------------------------------------------
/*
 * SLOT: Gnu was clicked (type changed) -> update labels, and deactivate if value==min_value
 */

void GnuTypeSelector::GnuTypeChanged()
{
  label_numbers_[type_]->setNum(label_numbers_[type_]->getNum()-1);
  
  if (label_numbers_[type_]->getNum()==0)
  {
    buttons_[type_]->setDisabled(1);
    buttons_[type_]->setOn(0);
    type_="";
    emit newType(type_);
  }
}

//-------------------------------------------------------------
/*
 * Enable/Disable all Buttons
 *
 * @param disabled
 */

void GnuTypeSelector::setButtonsDisabled(int disabled)
{
  ButtonMap::Iterator it;
  for ( it = buttons_.begin(); it != buttons_.end(); ++it )
  {
    buttons_[it.key().latin1()]->setDisabled(disabled);
    buttons_[it.key().latin1()]->setOn(0);

    if ((it.key().latin1()!="KillAll")&&(it.key().latin1()!="Pause")&&(it.key().latin1()!="FastForward"))
    {
      label_numbers_[it.key().latin1()]->setDisabled(disabled);

      if (buttons_[it.key().latin1()]->isEnabled())
      {
	if (label_numbers_[it.key().latin1()]->getNum()==0)
	{
	  buttons_[it.key().latin1()]->setDisabled(1);
	  label_numbers_[it.key().latin1()]->setDisabled(1);
	}
      }
    }
  }

  buttons_["KillAll"]->setDisabled(disabled);
  buttons_["Pause"]->setDisabled(disabled);
  buttons_["FastForward"]->setDisabled(disabled);
  label_numbers_["GnusLeft"]->setDisabled(disabled);
  label_numbers_["GnusSaved"]->setDisabled(disabled);
  label_gnussaved_image_->setDisabled(disabled);
  label_gnusleft_image_->setDisabled(disabled);
}

//-------------------------------------------------------------
/*
 * Change amount of gnus etc. (when loading new level)
 */

void GnuTypeSelector::setGnuConfig(int gnucount, int gnuminsave, int numblockers, int numdiggers, int numbashers, int numbuilders, int numfloaters, int numclimbers, int numbombers)
{
  label_numbers_["Blocker"]->setStartValue(numblockers);
  label_numbers_["Blocker"]->setNum(numblockers);

  label_numbers_["Digger"]->setStartValue(numdiggers);
  label_numbers_["Digger"]->setNum(numdiggers);

  label_numbers_["Basher"]->setStartValue(numbashers);
  label_numbers_["Basher"]->setNum(numbashers);

  label_numbers_["Builder"]->setStartValue(numbuilders);
  label_numbers_["Builder"]->setNum(numbuilders);

  label_numbers_["Floater"]->setStartValue(numfloaters);
  label_numbers_["Floater"]->setNum(numfloaters);

  label_numbers_["Climber"]->setStartValue(numclimbers);
  label_numbers_["Climber"]->setNum(numclimbers);
  
  label_numbers_["Bomber"]->setStartValue(numbombers);
  label_numbers_["Bomber"]->setNum(numbombers);

  label_numbers_["GnusLeft"]->setStartValue(gnucount);
  label_numbers_["GnusLeft"]->setNum(gnucount);
  buttons_["KillAll"]->setPixmap(*guipics_->getPic("Bomb"));

  label_numbers_["GnusSaved"]->setStartValue(0);
  label_numbers_["GnusSaved"]->setNum(0);
  label_numbers_["GnusSaved"]->setMinValue(gnuminsave);
  buttons_["Pause"]->setPixmap(*guipics_->getPic("Pause"));

  buttons_["FastForward"]->setPixmap(*guipics_->getPic("FastForward_inactive"));

  type_="";
  killall_pressed_=0;
  emit newType(type_);
}
