//gnuztimer.cpp - definition of the methods of GnuzTimer
/*
 * Copyright (C) 2003 Lukas Fraser <lukas.fraser@4cows.net>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 */

#include <gnuztimer.h>

//-------------------------------------------------------------
/*
 * constructor of GnuzTimer
 */

GnuzTimer::GnuzTimer():QTime()
{
  elapsed_time_=0;
  speed_=1;
  pause_=FALSE;
  start();
}

//-------------------------------------------------------------
/*
 * destructor of GnuzTimer
 */

GnuzTimer::~GnuzTimer()
{
}

//-------------------------------------------------------------
/*
 * Restart GnuzTimer
 */

void GnuzTimer::restart()
{
  elapsed_time_=0;
  speed_=1;
  pause_=FALSE;
  QTime::restart();
}

//-------------------------------------------------------------
/*
 * Pause GnuzTimer
 */

void GnuzTimer::pause()
{
  pause_=TRUE;
  elapsed_time_+=(elapsed()*speed_);
  QTime::restart();
}

//-------------------------------------------------------------
/*
 * Resume GnuzTimer
 */

void GnuzTimer::resume()
{
  pause_=FALSE;
  QTime::restart();
}

//-------------------------------------------------------------
/*
 * (de-)activate FF Mode => _speed_ times as fast (1==normal speed)
 */

void GnuzTimer::setFastForward(int speed)
{
  if (!pause_)
    elapsed_time_+=(elapsed()*speed_);

  QTime::restart();
  speed_=speed;
}

//-------------------------------------------------------------
/*
 * generate a QString timestamp m:ss
 *
 * @param seconds: create Timestamp from seconds
 * @return timestamp
 */

QString GnuzTimer::getTimeStamp(int seconds)
{
  QString output;
  char timestamp[10];

  if (!seconds)
    sprintf(timestamp, "%d:%.2d", getMinutes(), getSeconds()-(getMinutes()*60));
  else
    sprintf(timestamp, "%d:%.2d", seconds/60, seconds-((seconds/60)*60));

  output=timestamp;
  return(output);
}
