//level.h - declaration of the class Level
/*
 * Copyright (C) 2003 Lukas Fraser <lukas.fraser@4cows.net>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 */

#ifndef __level_h__
#define __level_h__

#include <qapplication.h>
#include <qcanvas.h>
#include <qimage.h>
#include <levelcanvasview.h>
#include <herd.h>
#include <levelpics.h>
#include <levelstats.h>
#include <levelconfig.h>
#include <gnutypeselector.h>
#include <guipics.h>
#include <gnuzconfig.h>
#include <gnuztimer.h>

//----------------------------------------------------------
/*
 * Level
 * 
 * 
 *
 */

class Level : public QObject
{
 Q_OBJECT

 signals:
  void setButtonsDisabled(bool);
  void hideLevelLoader();
  void updateScrollButtons();

 public slots:
  void GnusLeftChanged();
  void restartLevel();
  void loadLevel(QString filename);
  void cancelLoadLevel();
  void setAdvancePeriod(int period);
  void setFastForwardMode(bool active);

 protected:
  QCanvasSprite *levelpic_;
  QCanvas *level_;
  LevelCanvasView *level_view_;
  LevelPics *levelpics_;
  LevelStats *levelstats_;
  LevelConfig *levelconfig_;
  QWidget *parent_widget_;
  Herd *herd_;
  GnuTypeSelector *gnutypeselector_;
  QString current_level_;
  GuiPics *guipics_;
  GnuzConfig *config_;
  GnuzTimer *gnuztimer_;
  int level_advanceperiod_;

 public:
  Level(QWidget *parent, GnuTypeSelector *gnutypeselector, GnuzConfig *config, GuiPics *guipics);
  virtual ~Level();

  int getLevelHeight() { return(levelpics_->getLevelHeight()); }
  int getLevelWidth() { return(levelpics_->getLevelWidth()); }
  LevelCanvasView * getLevelView() { return(level_view_); }
  QScrollBar *getLevelScrollBar() { return(level_view_->horizontalScrollBar()); }
  Herd * getHerd() { return(herd_); }
  LevelCanvasView * getLevelCanvasView() { return(level_view_); }
};

#endif // __level_h__
