//levelpics.h - declaration of the class LevelPics
/*
 * Copyright (C) 2003 Lukas Fraser <lukas.fraser@4cows.net>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 */

#ifndef __levelpics_h__
#define __levelpics_h__

#include <qapplication.h>
#include <qcanvas.h>
#include <qimage.h>
#include <qpainter.h>
#include <gnutypeselectbutton.h>
#include <math.h>
#include <gnuzconfig.h>

//----------------------------------------------------------
/*
 * LevelPics
 * 
 * Stores LevelPics, LevelMask
 *
 */

class LevelPics
{
 protected:
  QImage levelmask_converted_;
  QImage *levelmask_;
  QCanvasPixmap *levelcanvaspixmap_;
  QCanvasPixmapArray *levelcanvaspixmaparray_;
  QCanvasSprite *levelsprite_;
  GnuzConfig *config_;

 public:
  LevelPics(GnuzConfig *config);
  virtual ~LevelPics();
  
  void loadPics(QPixmap levelpicfile, QImage levelmaskfile);
  void setLevelSprite(QCanvasSprite * levelsprite) { levelsprite_=levelsprite; }
  void updateLevelCanvasPixmapArray();
  bool drawRectLevel(int x, int y, int width, int height, QColor maskcolor, QColor maskfillcolor, QColor pixmapcolor);
  void drawCircleLevel(int x, int y, int radius, QColor color, QColor fillcolor, QColor pixmapcolor);
  bool isColorAtLevelMask(int x, int y, int width, int height, QColor color);
  bool isOtherColorAtLevelMask(int x, int y, int width, int height, QColor color1, QColor color2=QColor());
  QImage * getLevelMask() { return(levelmask_); }
  QCanvasPixmap * getLevelCanvasPixmap() { return(levelcanvaspixmap_); }
  QCanvasPixmapArray * getLevelCanvasPixmapArray() { return(levelcanvaspixmaparray_); }
  QCanvasSprite * getLevelSprite() { return(levelsprite_); }
  int getLevelHeight() { return(levelcanvaspixmap_->height()); }
  int getLevelWidth() { return(levelcanvaspixmap_->width()); }
};

#endif // __levelpics_h__
