//levelstats.cpp - definition of the methods of LevelStats
/*
 * Copyright (C) 2003 Lukas Fraser <lukas.fraser@4cows.net>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 */

#include <levelstats.h>

//-------------------------------------------------------------
/*
 * constructor of LevelStats
 *
 * @param gnutypeselector
 */

LevelStats::LevelStats(GnuTypeSelector * gnutypeselector, LevelConfig * levelconfig, GnuzConfig * config, GuiPics *guipics, GnuzTimer *gnuztimer)
{
  config_=config;
  levelconfig_=levelconfig;
  gnutypeselector_=gnutypeselector;
  guipics_=guipics;
  gnuztimer_=gnuztimer;
  
  QList<QPixmap> picslist;
  QList<QPoint> pointlist;
  QPoint temppoint;
  
  picslist.append(guipics_->getPic("LevelStats"));
  pointlist.append(&temppoint);

  statscanvaspixmaparray_ = new QCanvasPixmapArray(picslist, pointlist);
  statscanvaspixmap_ = statscanvaspixmaparray_->image(0);
  
  v_border_=14;
  h_border_=5;
}

//-------------------------------------------------------------
/*
 * destructor of LevelStats
 */

LevelStats::~LevelStats()
{
  delete statscanvaspixmaparray_;
  delete statscanvaspixmap_;
}

//-------------------------------------------------------------
/*
 * generate stats pixmap
 *
 * @return stats pixmap
 */

QCanvasPixmapArray * LevelStats::getStats()
{
  bool won=FALSE;
  char output[30];
  QString buttonnames[]={"Blocker", "Digger", "Basher", "Builder", "Floater", "Climber", "Bomber"};
  QString title;
  int skills_left=0;

  if (gnutypeselector_->getLabel("GnusSaved")->getNum()>=gnutypeselector_->getLabel("GnusSaved")->getMinValue())
    won=TRUE;

  //Load background image and display it
  QList<QPixmap> picslist;
  QList<QPoint> pointlist;
  QPoint temppoint;
  picslist.append(guipics_->getPic("LevelStats"));
  pointlist.append(&temppoint);

  statscanvaspixmaparray_ = new QCanvasPixmapArray(picslist, pointlist);
  statscanvaspixmap_ = statscanvaspixmaparray_->image(0);
  
  QPainter painter(statscanvaspixmap_);
  painter.setFont(QFont("Cronyx-Helvetica", 12, QFont::Bold));

  //Display 'Level Complete' or 'Game Over' image
  if (won)
    title="LevelComplete";
  else
    title="GameOver";

  painter.drawPixmap((240-guipics_->getPic("LevelStats"+title)->width())/2, v_border_, *guipics_->getPic("LevelStats"+title));

  //Display gnu skills
  for (int i=0; i<=6;i++)
  {
    painter.drawPixmap(h_border_, v_border_+26+(i*20), *gnutypeselector_->getButton(buttonnames[i])->pixmap());

    skills_left+=gnutypeselector_->getLabel(buttonnames[i])->getNum();
    sprintf(output, "%d/%d", gnutypeselector_->getLabel(buttonnames[i])->getNum(), gnutypeselector_->getLabel(buttonnames[i])->getStartValue());

    painter.setPen(Qt::yellow); //No skills used
    if (gnutypeselector_->getLabel(buttonnames[i])->getNum()==gnutypeselector_->getLabel(buttonnames[i])->getStartValue()) //Some used, a few left
      painter.setPen(Qt::green);
    if (gnutypeselector_->getLabel(buttonnames[i])->getNum()<=gnutypeselector_->getLabel(buttonnames[i])->getMinValue()) //All used
      painter.setPen(Qt::red);
    if (gnutypeselector_->getLabel(buttonnames[i])->getStartValue()==0) //None available
      painter.setPen(Qt::white);

    painter.drawText(h_border_+4+gnutypeselector_->getButton(buttonnames[i])->pixmap()->width(), v_border_+43+(i*20), output);
  }

  //GnusSaved etc stats
  painter.setPen(Qt::white);
  sprintf(output, "%d saved", gnutypeselector_->getLabel("GnusSaved")->getNum());
  painter.drawText(h_border_+109, v_border_+38, output);
  sprintf(output, "%d killed", gnutypeselector_->getLabel("GnusLeft")->getStartValue()-gnutypeselector_->getLabel("GnusSaved")->getNum());
  painter.drawText(h_border_+109, v_border_+55, output);
  sprintf(output, "%d of %d needed", gnutypeselector_->getLabel("GnusSaved")->getMinValue(), gnutypeselector_->getLabel("GnusLeft")->getStartValue());
  painter.drawText(h_border_+109, v_border_+76, output);

  //Time stats
  if (gnuztimer_->getSeconds()>levelconfig_->getLevelData("LevelParTime").toInt())
    painter.setPen(Qt::red);
  else
    painter.setPen(Qt::green);

  if (won)
    sprintf(output, "%s", gnuztimer_->getTimeStamp().latin1());
  else
  {
    painter.setPen(Qt::yellow);
    sprintf(output, "(%s)", gnuztimer_->getTimeStamp().latin1());
  }

  painter.drawText(h_border_+138, v_border_+102, output);
  painter.setPen(Qt::white);
  painter.drawText(h_border_+138, v_border_+118, "Par: "+gnuztimer_->getTimeStamp(levelconfig_->getLevelData("LevelParTime").toInt()));


  //Score stats
  if (won)
  {
    int score=0;
    score+=(gnutypeselector_->getLabel("GnusSaved")->getNum()-gnutypeselector_->getLabel("GnusSaved")->getMinValue())*200; //200 points per extra saved gnu
    score+=skills_left*50; //50 points per saved skill
    if (gnuztimer_->getSeconds()<levelconfig_->getLevelData("LevelParTime").toInt()) //10 points per second faster than par time
      score+=(levelconfig_->getLevelData("LevelParTime").toInt()-gnuztimer_->getSeconds())*10;
    
    painter.setFont(QFont("Cronyx-Helvetica", 14, QFont::Bold));
    sprintf(output, "Score: %d", score);
    painter.drawText(h_border_+138, v_border_+156, output);
  }
  else
    painter.drawText(h_border_+138, v_border_+156, "Try again!");

  painter.end();

  return(statscanvaspixmaparray_);
}

